#-----------------------------------------------------------------------------
# Version: 1.1.0
# Compatible: SketchUp 7 (PC)
#             (other versions untested)
#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

require 'sketchup.rb'

#-----------------------------------------------------------------------------


module TT_Mat_Replace
	
	unless file_loaded?('tt_mat_replace.rb')
		plugins_menu = UI.menu('Tools')
		plugins_menu.add_item('Material Replacer')	{ self.activate_tool }
	end
	
	def self.activate_tool
		my_tool = MatReplacer.new
		Sketchup.active_model.tools.push_tool(my_tool)
	end

	class MatReplacer
	
		def initialize
			@picked = nil
			@m_org = nil
			@state = 0
			
			path = 'Plugins/TT_MatRep'
			c_dropper		= Sketchup.find_support_file('dropper.png', path)
			c_dropper_err	= Sketchup.find_support_file('dropper_invalid.png', path)
			@c_dropper		= (c_dropper) ? UI.create_cursor(c_dropper, 2, 29) : 0
			@c_dropper_err	= (c_dropper_err) ? UI.create_cursor(c_dropper_err, 2, 29) : 0
		end	
		
		def activate
			@picked = nil
			@m_org = nil
			@state = 0
			@pos = [0,0,0]
			@drawn = false
			@same_volume=false
			Sketchup.set_status_text('Pick a material to replace.')
			@src_vol = nil 
			@debug = nil
		end
		
		def deactivate(view)
			view.invalidate if @drawn
		end
		
		def onMouseMove(flags, x, y, view)
			ph = view.pick_helper
			ph.do_pick(x, y)
			
			picked = (ph.picked_face.nil?) ? (ph.picked_edge.nil?) ? ph.best_picked : ph.picked_edge : ph.picked_face
			
			index = 0
			for i in 1..ph.count
				if ph.element_at(i) == picked
					index = i
					break 
				end
			end
			path = ph.path_at(index)
			t = ph.transformation_at(index)
			
			if path.nil?
				@picked = nil
			else
				@picked = nil
				# Get picked face global normal
				n = ( picked.is_a?(Sketchup::Face) ) ? picked.normal.transform(t).normalize : nil

				# Travel down the path to work out the visible material
				path.each { |e|
					next unless e.respond_to?(:material)
					if e.is_a?(Sketchup::Face)
						c = Sketchup.active_model.active_view.camera.direction
						m = (c % n > 0) ? e.back_material : e.material
						next if m.nil?
						@picked = m
						if e.class!=Sketchup::Edge and e.class!=Sketchup::Face
						@src_vol= e.volume
						end
					else
						next if e.material.nil?
						@picked = e.material
						if e.class!=Sketchup::Edge and e.class!=Sketchup::Face
						@src_vol= e.volume
						end
					end
				}
				
				# DEBUG
				#@path = path
				#@debug = n
			end
			
			@pos = [x,y,0]
			view.invalidate
		end
		
		def onLButtonUp(flags, x, y, view)
			if @state == 0
				@m_org = @picked
				@state = 1
				
				o_name = (@m_org.nil?) ? 'Default' : @m_org.display_name
				Sketchup.set_status_text("Pick another material to replace #{o_name}.")
			else
				return if @picked == @m_org
				
				model = Sketchup.active_model
				
				if @src_vol>0
					if UI.messagebox("apply only to element with same volume?", MB_YESNO)==6
						@same_volume=true
					else 
						@same_volume=false
					end
				end
				TT_Mat_Replace.start_operation('Replace Materials')
				Sketchup.set_status_text('Replacing materials. Please wait...')
				printf("source volume is: %s\n",@src_vol)
				model.entities.each { |e|
					if e.respond_to?(:material)
						if @same_volume and e.class!=Sketchup::Edge and e.class!=Sketchup::Face and e.volume>0
								e.material = @picked if e.material == @m_org and e.volume==@src_vol
						else
						e.material = @picked if e.material == @m_org
						end
					end
					if e.respond_to?(:back_material)
						if @same_volume and e.class!=Sketchup::Edge and e.class!=Sketchup::Face and e.volume>0
							e.back_material = @picked if e.back_material == @m_org and e.volume==@src_vol
						else
						e.back_material = @picked if e.back_material == @m_org
						end
					end
				}
				model.definitions.each { |d|
					next if d.image?
					d.entities.each { |e|
						if e.respond_to?(:material)
						if @same_volume and e.class!=Sketchup::Edge and e.class!=Sketchup::Face and e.volume>0
								e.material = @picked if e.material == @m_org and e.volume==@src_vol
						else
						e.material = @picked if e.material == @m_org
						end
					end
					if e.respond_to?(:back_material)
						if @same_volume and e.class!=Sketchup::Edge and e.class!=Sketchup::Face and e.volume>0
							e.back_material = @picked if e.back_material == @m_org and e.volume==@src_vol
						else
						e.back_material = @picked if e.back_material == @m_org
						end
					end
					}
				}
					
				model.commit_operation
				
				activate
			end
		end
		
		def draw(view)
			pos = @pos
			pos[0] += 20
			pos[1] -= 20
			if @state == 0
				#o_name = (@m_org.nil?) : 'Default' : @m_org.display_name
				p_name = (@picked.nil?) ? 'Default' : @picked.display_name
				str = "Replace #{p_name} ..."
				view.draw_text(pos, str)
			else
				o_name = (@m_org.nil?) ? 'Default' : @m_org.display_name
				p_name = (@picked.nil?) ? 'Default' : @picked.display_name
				str = "Replace #{o_name}\nwith #{p_name}"
				view.draw_text(pos, str)
			end
			
			# DEBUG
			#pos[1] += 40
			#view.draw_text(pos, "#{@debug.inspect}\n#{@path.inspect}")
			
			@drawn = true
		end
		
		def onSetCursor
			if @state == 1 && @picked == @m_org
				UI.set_cursor(@c_dropper_err)
			else
				UI.set_cursor(@c_dropper)
			end
		end

	end # class
	
	### HELPER METHODS ### ---------------------------------------------------
	
	def self.start_operation(name)
		model = Sketchup.active_model
		# Make use of the SU7 speed boost with start_operation while
		# making sure it works in SU6.
		if Sketchup.version.split('.')[0].to_i >= 7
			model.start_operation(name, true)
		else
			model.start_operation(name)
		end
	end
	
end # module

file_loaded('tt_mat_replace.rb')